@@ -289,10 +289,9 @@ void file_open_ok_callback(GtkWidget *widget, gpointer data)
         gtk_widget_destroy(ggv->file_sel);
         ggv->file_sel = NULL;
 
-        if ((ggv->loaded = load_gs(ggv, file))) {
-                recent_add (file);
-                recent_update (ggv);
-        }
+        if ((ggv->loaded = load_gs(ggv, file)))
+                recent_update();
+
         g_free(file);
 }
 
@@ -539,10 +538,15 @@ void print_callback(GtkWidget *widget, gpointer data)
 {
         ggv_window *ggv = (ggv_window *) data;
         gchar *print_command;
+        GtkGS *gs = GTK_GS(ggv->gs);
 
         if(ggv->loaded) {
+                /* make sure we print the _uncompressed_ file in case it
+                   the loaded one was compressed! */
                 print_command = g_strdup_printf(gs_print_cmd,
-                                                GTK_GS(ggv->gs)->gs_filename);
+                                                (gs->gs_filename_unc != NULL?
+                                                 gs->gs_filename_unc:
+                                                 gs->gs_filename));
                 if(print_command) {
                         gnome_execute_shell(NULL, print_command);
                         g_free(print_command);
@@ -659,6 +663,14 @@ void save_callback(GtkWidget *widget, gpointer data) {
         if (ggv->loaded) {
                 if(ggv->save_file_sel)
                         return;
+
+                if(gtk_gs_count_marked_pages(GTK_GS(ggv->gs)) == 0) {
+                        error_msg = g_strdup(_("There are no marked pages."));
+                        error_message(ggv, error_msg);
+                        return;
+                }
+        
+
                 
                 ggv->save_file_sel = gtk_file_selection_new(_("ggv: Save marked pages"));
                 gtk_signal_connect(GTK_OBJECT(ggv->save_file_sel), "delete_event",
