@@ -95,9 +95,9 @@ gtk_scrollpane_init(GtkScrollpane * sp)
   sp->Xadjustment = NULL;
   sp->Yadjustment = NULL;
   sp->moving = 0;
+  sp->in_slider = FALSE;
   sp->trough_width = SCROLLPANE_DEFAULT_SIZE;
   sp->trough_height = SCROLLPANE_DEFAULT_SIZE;
-
 }
 
 static void
@@ -174,7 +174,6 @@ gtk_scrollpane_realize(GtkWidget * widget)
   g_return_if_fail(GTK_IS_SCROLLPANE(widget));
 
   sp = GTK_SCROLLPANE(widget);
-  GTK_WIDGET_SET_FLAGS(widget, GTK_REALIZED);
 
   /* 
    * set up the widget
@@ -211,6 +210,8 @@ gtk_scrollpane_realize(GtkWidget * widget)
 			      attributes_mask);
 
   gtk_scrollpane_update_slider_size(sp);
+
+  GTK_WIDGET_SET_FLAGS(widget, GTK_REALIZED);
   widget->style = gtk_style_attach(widget->style, widget->window);
 
   gdk_window_set_user_data(sp->trough, widget);
@@ -312,26 +313,15 @@ gtk_real_scrollpane_draw_trough(GtkScrollpane * sp)
 
   if (sp->trough)
     {
-      gtk_draw_shadow(GTK_WIDGET(sp)->style,
-		      sp->trough,
-		      GTK_STATE_NORMAL,
-		      GTK_SHADOW_IN,
-		      0, 0, -1, -1);
-
-      if (GTK_WIDGET_HAS_FOCUS(sp))
-	gdk_draw_rectangle
-	  (GTK_WIDGET(sp)->window,
-	   GTK_WIDGET(sp)->style->black_gc,
-	   FALSE, 0, 0,
-	   GTK_WIDGET(sp)->allocation.width - 1,
-	   GTK_WIDGET(sp)->allocation.height - 1);
-      else if (sp->trough != GTK_WIDGET(sp)->window)
-	gdk_draw_rectangle
-	  (GTK_WIDGET(sp)->window,
-	   GTK_WIDGET(sp)->style->bg_gc[GTK_STATE_NORMAL],
-	   FALSE, 0, 0,
-	   GTK_WIDGET(sp)->allocation.width - 1,
-	   GTK_WIDGET(sp)->allocation.height - 1);
+            gtk_paint_box (GTK_WIDGET (sp)->style, sp->trough,
+                           GTK_STATE_ACTIVE, GTK_SHADOW_IN,
+                           NULL, GTK_WIDGET(sp), "trough",
+                           0, 0, -1, -1);
+            if (GTK_WIDGET_HAS_FOCUS (sp))
+                    gtk_paint_focus (GTK_WIDGET (sp)->style,
+                                     sp->trough,
+                                     NULL, GTK_WIDGET(sp), "trough",
+                                     0, 0, -1, -1);
     }
 }
 
@@ -345,45 +335,43 @@ gtk_real_scrollpane_draw_slider(GtkScrollpane * sp)
 
   if (sp->slider)
     {
-      state_type = GTK_STATE_NORMAL;
+      if (sp->in_slider || sp->moving)
+	state_type = GTK_STATE_PRELIGHT;              
+      else
+        state_type = GTK_STATE_NORMAL;
 
-      gtk_style_set_background(GTK_WIDGET(sp)->style,
-			       sp->slider,
-			       state_type);
-      gdk_window_clear(sp->slider);
-
-      gtk_draw_shadow(GTK_WIDGET(sp)->style,
-		      sp->slider, state_type,
-		      GTK_SHADOW_OUT,
-		      0, 0, -1, -1);
+      gtk_paint_box (GTK_WIDGET (sp)->style, sp->slider,
+		     state_type, GTK_SHADOW_OUT,
+		     NULL, GTK_WIDGET (sp), "slider",
+		     0, 0, -1, -1);
     }
 }
 
 static void
 gtk_scrollpane_size_request(GtkWidget * widget, GtkRequisition * requisition)
 {
+  GtkScrollpane *sp = GTK_SCROLLPANE(widget);
+
   g_return_if_fail(widget != NULL);
   g_return_if_fail(GTK_IS_SCROLLPANE(widget));
   g_return_if_fail(requisition != NULL);
 
-/*  
- * requisition->width = 
- * (GTK_CONTAINER (widget)->border_width + 
- * GTK_WIDGET (widget)->style->klass->xthickness) * 2 +
- * widget->allocation.width;
- * requisition->height = 
- * (GTK_CONTAINER (widget)->border_width + 
- * GTK_WIDGET (widget)->style->klass->ythickness) * 2 +
- * widget->allocation.height;
- */
-/*  
- * requisition->width = SCROLLPANE_DEFAULT_SIZE;
- * requisition->height = SCROLLPANE_DEFAULT_SIZE;
- */
+  if(sp->trough_width <= 0)
+          sp->trough_width = SCROLLPANE_DEFAULT_SIZE;
+  if(sp->trough_height <= 0)
+          sp->trough_height = SCROLLPANE_DEFAULT_SIZE;
+
+  requisition->width = 
+          widget->style->klass->xthickness*2 +
+          sp->trough_width;
+
+  requisition->height =
+          widget->style->klass->ythickness*2 +
+          sp->trough_height;
 }
 
 static void
-gtk_scrollpane_size_allocate(GtkWidget * widget, GtkAllocation * allocation)
+gtk_scrollpane_size_allocate(GtkWidget * widget, GtkAllocation *allocation)
 {
   g_return_if_fail(widget != NULL);
   g_return_if_fail(GTK_IS_SCROLLPANE(widget));
@@ -405,10 +393,12 @@ gtk_scrollpane_update_slider_size(GtkScrollpane * sp)
   g_return_if_fail(GTK_IS_SCROLLPANE(sp));
 
   widget = GTK_WIDGET(sp);
-  width = widget->allocation.width;
-  height = widget->allocation.height;
   x = widget->allocation.x;
   y = widget->allocation.y;
+  width = widget->allocation.width;
+  height = widget->allocation.height;
+  width -= GTK_WIDGET(widget)->style->klass->xthickness*2;
+  height -= GTK_WIDGET(widget)->style->klass->ythickness*2;
 
   sp->trough_width = width;
   sp->trough_height = height;
@@ -416,7 +406,7 @@ gtk_scrollpane_update_slider_size(GtkScrollpane * sp)
   /* we also do some sanity checking to guarentee that nothing weird has been passed in */
   /* (borrowed in spirit from gtk?scrollbar) */
   x_offset = widget->style->klass->xthickness;
-  sp->slider_width = width - widget->style->klass->xthickness - widget->style->klass->xthickness;
+  sp->slider_width = width;
   if (sp->Xadjustment)
     {
       if ((sp->Xadjustment->page_size > 0) && (sp->Xadjustment->lower < sp->Xadjustment->upper))
@@ -424,30 +414,26 @@ gtk_scrollpane_update_slider_size(GtkScrollpane * sp)
 	  if (sp->Xadjustment->page_size < (sp->Xadjustment->upper - sp->Xadjustment->lower))
 	    {
 	      /* calculate the X offset... */
-	      x_offset = ((gfloat) (width - widget->style->klass->xthickness - widget->style->klass->xthickness))
-		* ((sp->Xadjustment->value - sp->Xadjustment->page_size / 2 - sp->Xadjustment->lower) /
-		   (sp->Xadjustment->upper - sp->Xadjustment->lower)) + widget->style->klass->xthickness;
+	      x_offset += ((gfloat) (width))
+                      * ((sp->Xadjustment->value - sp->Xadjustment->page_size / 2 - sp->Xadjustment->lower) /
+                         (sp->Xadjustment->upper - sp->Xadjustment->lower));
 
 	      /* scale the width appropriately */
-	      sp->slider_width = ((gfloat) width) * sp->Xadjustment->page_size /
-		(sp->Xadjustment->upper - sp->Xadjustment->lower);
+	      sp->slider_width = 1 + ((gfloat) width)*sp->Xadjustment->page_size/(sp->Xadjustment->upper - sp->Xadjustment->lower);
 	      if (sp->slider_width < sp->min_slider_width)
 		sp->slider_width = sp->min_slider_width;
-	      if (sp->slider_width + widget->style->klass->xthickness + widget->style->klass->xthickness > width)
+	      if (sp->slider_width > width)
 		{
 		  /* this takes care of rounding errors i think.  occasionally, there's a 1 pixel error */
 		  x_offset = widget->style->klass->xthickness;
-		  sp->slider_width = width - widget->style->klass->xthickness - widget->style->klass->xthickness;
+		  sp->slider_width = width;
 		}
 	    }
 	}
-      /* FIXME: This is a hack to prevent an error somewhere in the above code ): */
-      if (sp->slider_width + x_offset > width - widget->style->klass->xthickness)
-	sp->slider_width = width - widget->style->klass->xthickness - x_offset;
     }
 
   y_offset = widget->style->klass->ythickness;
-  sp->slider_height = height - widget->style->klass->ythickness - widget->style->klass->ythickness;
+  sp->slider_height = height;
   if (sp->Yadjustment)
     {
       if ((sp->Yadjustment->page_size > 0) && (sp->Yadjustment->lower < sp->Yadjustment->upper))
@@ -455,34 +441,34 @@ gtk_scrollpane_update_slider_size(GtkScrollpane * sp)
 	  if (sp->Yadjustment->page_size < (sp->Yadjustment->upper - sp->Yadjustment->lower))
 	    {
 	      /* calculate the Y offset... */
-	      y_offset = ((gfloat) (height - widget->style->klass->ythickness - widget->style->klass->ythickness))
-		* ((sp->Yadjustment->value - sp->Yadjustment->page_size / 2 - sp->Yadjustment->lower) /
-		   (sp->Yadjustment->upper - sp->Yadjustment->lower)) + widget->style->klass->ythickness;
+	      y_offset += ((gfloat) (height))
+                      * ((sp->Yadjustment->value - sp->Yadjustment->page_size / 2 - sp->Yadjustment->lower) /
+                         (sp->Yadjustment->upper - sp->Yadjustment->lower));
 
 	      /* scale the height appropriately */
-	      sp->slider_height = ((gfloat) height) * sp->Yadjustment->page_size /
-		(sp->Yadjustment->upper - sp->Yadjustment->lower);
+	      sp->slider_height = 1 + ((gfloat) height)*sp->Yadjustment->page_size/(sp->Yadjustment->upper - sp->Yadjustment->lower);
 	      if (height < sp->min_slider_height)
 		sp->slider_height = sp->min_slider_height;
-	      if (sp->slider_height + widget->style->klass->ythickness + widget->style->klass->ythickness > height)
+	      if (sp->slider_height > height)
 		{
 		  /* as per above... */
 		  y_offset = widget->style->klass->ythickness;
-		  sp->slider_height = height - widget->style->klass->ythickness - widget->style->klass->ythickness;
+		  sp->slider_height = height;
 		}
 	    }
 	}
-      /* FIXME: This is a hack to prevent an error somewhere in the above code ): */
-      if (sp->slider_height + y_offset > height - widget->style->klass->ythickness)
-	sp->slider_height = height - widget->style->klass->ythickness - y_offset;
     }
 
+  /* FIXME: This makes the scrollpane grow indefinitely */
   if (widget->window)
-    gdk_window_move_resize(widget->window,
-			   x, y, width, height);
+          gdk_window_move_resize(widget->window,
+                                 x, y,
+                                 widget->allocation.width, widget->allocation.height);
+
   if (sp->slider)
-    gdk_window_move_resize(sp->slider,
-			   x_offset, y_offset, sp->slider_width, sp->slider_height);
+          gdk_window_move_resize(sp->slider,
+                                 x_offset, y_offset,
+                                 sp->slider_width, sp->slider_height);
 }
 
 static void
@@ -525,27 +511,15 @@ gtk_scrollpane_expose(GtkWidget * widget, GdkEventExpose * event)
 
   /* where did the expose occur??? */
   /* if it did in the trough, then... */
-  if (event->window == sp->trough)
-    {
-      gint                xt = widget->style->klass->xthickness;
-      gint                yt = widget->style->klass->ythickness;
-
-      if (!((event->area.x >= xt) &&
-	    (event->area.y >= yt) &&
-	    (event->area.x + event->area.width <=
-	     widget->allocation.width - xt) &&
-	    (event->area.y + event->area.height <=
-	     widget->allocation.height - xt)))
-	gtk_scrollpane_draw_trough(sp);
-    }
-  else if (event->window == widget->window)
-    {
-      gtk_scrollpane_draw_background(sp);
-    }
-  else if (event->window == sp->slider)
-    {
-      gtk_scrollpane_draw_slider(sp);
-    }
+  if (event->window == sp->trough) {
+          gtk_scrollpane_draw_trough(sp);
+  }
+  else if (event->window == widget->window) {
+          gtk_scrollpane_draw_background(sp);
+  }
+  else if (event->window == sp->slider) {
+          gtk_scrollpane_draw_slider(sp);
+  }
   return FALSE;
 }
 
@@ -628,57 +602,155 @@ gtk_scrollpane_new(GtkAdjustment * Xadjustment,
   return GTK_WIDGET(retval);
 }
 gboolean 
-gtk_scrollpane_step_up(GtkScrollpane * sp)
+gtk_scrollpane_goto_edge(GtkScrollpane * sp, gint vertical, gint horizontal)
 {
   g_return_val_if_fail(sp != NULL, FALSE);
+  g_return_if_fail(GTK_IS_SCROLLPANE(sp));
 
-  if (sp->Yadjustment->value - sp->Yadjustment->page_size / 2 - EPSILON <= sp->Yadjustment->lower)
-    return FALSE;
-  sp->Yadjustment->value -= 0.75 * (sp->Yadjustment->page_size);
-  if (sp->Yadjustment->value - sp->Yadjustment->page_size / 2 - EPSILON <= sp->Yadjustment->lower)
+  switch (vertical) {
+  case GTK_SCROLLPANE_GOTOEDGE_LOWER:
     sp->Yadjustment->value = sp->Yadjustment->lower + sp->Yadjustment->page_size / 2;
-  gtk_adjustment_value_changed(sp->Yadjustment);
-  return TRUE;
-}
-gboolean 
-gtk_scrollpane_step_down(GtkScrollpane * sp)
-{
-  g_return_val_if_fail(sp != NULL, FALSE);
-
-  if (sp->Yadjustment->value + sp->Yadjustment->page_size / 2 + EPSILON >= sp->Yadjustment->upper)
-    return FALSE;
-  sp->Yadjustment->value += 0.75 * (sp->Yadjustment->page_size);
-  if (sp->Yadjustment->value + sp->Yadjustment->page_size / 2 + EPSILON >= sp->Yadjustment->upper)
+    gtk_adjustment_value_changed(sp->Yadjustment);
+    break;
+  case GTK_SCROLLPANE_GOTOEDGE_UPPER:
     sp->Yadjustment->value = sp->Yadjustment->upper - sp->Yadjustment->page_size / 2;
-  gtk_adjustment_value_changed(sp->Yadjustment);
-  return TRUE;
-}
-gboolean 
-gtk_scrollpane_step_right(GtkScrollpane * sp)
-{
-  g_return_val_if_fail(sp != NULL, FALSE);
+    break;
+  case GTK_SCROLLPANE_GOTOEDGE_NONE:
+    break;
+  default:
+    g_assert(0); /* Illegal parameter error */
+  }
+  if (vertical != GTK_SCROLLPANE_GOTOEDGE_NONE)
+    gtk_adjustment_value_changed(sp->Yadjustment);
 
-  if (sp->Xadjustment->value + sp->Xadjustment->page_size / 2 + EPSILON >= sp->Xadjustment->upper)
-    return FALSE;
-  sp->Xadjustment->value += 0.75 * (sp->Xadjustment->page_size);
-  if (sp->Xadjustment->value + sp->Xadjustment->page_size / 2 + EPSILON >= sp->Xadjustment->upper)
-    sp->Xadjustment->value = sp->Xadjustment->upper - sp->Xadjustment->page_size / 2;
-  gtk_adjustment_value_changed(sp->Xadjustment);
-  return TRUE;
-}
-gboolean 
-gtk_scrollpane_step_left(GtkScrollpane * sp)
-{
-  g_return_val_if_fail(sp != NULL, FALSE);
 
-  if (sp->Xadjustment->value - sp->Xadjustment->page_size / 2 - EPSILON <= sp->Xadjustment->lower)
-    return FALSE;
-  sp->Xadjustment->value -= 0.75 * (sp->Xadjustment->page_size);
-  if (sp->Xadjustment->value - sp->Xadjustment->page_size / 2 - EPSILON <= sp->Xadjustment->lower)
+  switch (vertical) {
+  case GTK_SCROLLPANE_GOTOEDGE_LOWER:
     sp->Xadjustment->value = sp->Xadjustment->lower + sp->Xadjustment->page_size / 2;
-  gtk_adjustment_value_changed(sp->Xadjustment);
+    gtk_adjustment_value_changed(sp->Xadjustment);
+    break;
+  case GTK_SCROLLPANE_GOTOEDGE_UPPER:
+    sp->Xadjustment->value = sp->Xadjustment->upper - sp->Xadjustment->page_size / 2;
+    break;
+  case GTK_SCROLLPANE_GOTOEDGE_NONE:
+    break;
+  default:
+    g_assert(0); /* Illegal parameter error */
+  }
+  if (vertical != GTK_SCROLLPANE_GOTOEDGE_NONE)
+    gtk_adjustment_value_changed(sp->Xadjustment);
+
   return TRUE;
 }
+
+
+gboolean 
+gtk_scrollpane_step(GtkScrollpane * sp, gint direction, gboolean dowrap)
+{
+  GtkAdjustment *MainAdj; /* We will move this adjustment */
+  GtkAdjustment *SecoAdj; /* And this _only_ if we can't move MainAdj (ie. we're edge)
+			     and there is wrapping */
+
+  gboolean MoveHorizontal; /* Positive if we move horizontal */
+  gboolean DirectionFlag;  /* Positive if we move towards upper */
+  g_return_val_if_fail(sp != NULL, FALSE);
+  g_return_if_fail(GTK_IS_SCROLLPANE(sp));
+
+
+#define CHECK_THERE_IS_NO_LOWER_SPACE(adj) ((adj)->value - ((adj)->page_size / 2 + EPSILON) <= (adj)->lower)
+#define CHECK_THERE_IS_NO_UPPER_SPACE(adj) ((adj)->value + ((adj)->page_size / 2 + EPSILON) >= (adj)->upper)
+
+#define CHECK_THERE_IS_NO_SPACE_FOR_STEP(adj,dir) (dir?CHECK_THERE_IS_NO_UPPER_SPACE(adj):\
+						   CHECK_THERE_IS_NO_LOWER_SPACE(adj))
+
+  /* To make code more readable, we make a macro */
+
+#define ADVANCE_TOWARDS_LOWER(adj)         (adj->value -= 0.75 * (adj->page_size))
+#define ADVANCE_TOWARDS_UPPER(adj)         (adj->value += 0.75 * (adj->page_size))
+
+#define ADVANCE_STEP(adj,dir)      (dir?ADVANCE_TOWARDS_UPPER(adj):ADVANCE_TOWARDS_LOWER(adj))
+
+#define MOVE_TO_LOWER_EDGE(adj)    (adj->value = adj->lower + adj->page_size / 2)
+#define MOVE_TO_UPPER_EDGE(adj)    (adj->value = adj->upper - adj->page_size / 2)
+
+    /* if upper is 1 goto upper, otherwise to lower */
+
+#define MOVE_TO_EDGE(adj,upper) (upper?MOVE_TO_UPPER_EDGE(adj):MOVE_TO_LOWER_EDGE(adj))
+
+  /* These variables make our life easier */
+
+  switch (direction) {
+  case GTK_SCROLLPANE_SCROLL_RIGHT:
+    MoveHorizontal = TRUE;
+    DirectionFlag = TRUE;
+    break;
+  case GTK_SCROLLPANE_SCROLL_LEFT:
+    MoveHorizontal = TRUE;
+    DirectionFlag = FALSE;
+    break;
+  case GTK_SCROLLPANE_SCROLL_DOWN:
+    MoveHorizontal = FALSE;
+    DirectionFlag = TRUE;
+    break;
+  case GTK_SCROLLPANE_SCROLL_UP:
+    MoveHorizontal = FALSE;
+    DirectionFlag = FALSE;
+    break;
+  default:
+    g_assert(0); /* Illegal parameter error */
+  }
+    
+  if (MoveHorizontal) {
+    MainAdj = sp->Xadjustment;
+    SecoAdj = sp->Yadjustment;
+  }
+  else {
+    MainAdj = sp->Yadjustment;
+    SecoAdj = sp->Xadjustment;
+  }
+
+  if (CHECK_THERE_IS_NO_SPACE_FOR_STEP(MainAdj, DirectionFlag)) {
+    g_print("There was no space to move\n");
+    if (!dowrap)
+      return FALSE;
+
+    g_print("There was no space to move, but we do wrapping\n");
+    /* We do wrapping */
+
+    /* Move in the oposite axis */
+    if (CHECK_THERE_IS_NO_SPACE_FOR_STEP(SecoAdj, DirectionFlag)) {
+      /* there is no place to move, we need a new page */
+      return FALSE;
+    }
+    ADVANCE_STEP(SecoAdj,DirectionFlag);
+    
+    if (CHECK_THERE_IS_NO_SPACE_FOR_STEP(SecoAdj,DirectionFlag)) {
+      /* We move it too far, lets move it to the edge */
+      MOVE_TO_EDGE(SecoAdj,DirectionFlag);
+    }
+    /* now move to edge (other axis) in oposite direction */
+    MOVE_TO_EDGE(MainAdj,!DirectionFlag);
+    gtk_adjustment_value_changed(SecoAdj);
+    return TRUE;
+  }
+  
+  /* Now we know we can move in the direction sought */
+  
+  
+  ADVANCE_STEP(MainAdj,DirectionFlag);
+  
+  if (CHECK_THERE_IS_NO_SPACE_FOR_STEP(MainAdj,DirectionFlag)) {
+    /* We move it too far, lets move it to the edge */
+
+    MOVE_TO_EDGE(MainAdj,DirectionFlag);
+  }
+  gtk_adjustment_value_changed(MainAdj);
+  
+  return TRUE;
+
+}
+
+
 static void
 gtk_scrollpane_adjustment_changed(GtkAdjustment * adjustment, gpointer data)
 {
@@ -711,13 +783,27 @@ gtk_scrollpane_key_press(GtkWidget * widget, GdkEventKey * event)
 static              gint
 gtk_scrollpane_enter_notify(GtkWidget * widget, GdkEventCrossing * event)
 {
-  return TRUE;
+  GtkScrollpane *pane = GTK_SCROLLPANE(widget);
+
+  if(event->window == pane->slider)
+    {
+      pane->in_slider = TRUE;
+      gtk_scrollpane_draw_slider(pane);
+    }
+
+  return FALSE;
 }
 
 static              gint
 gtk_scrollpane_leave_notify(GtkWidget * widget, GdkEventCrossing * event)
 {
-  return TRUE;
+  GtkScrollpane *pane = GTK_SCROLLPANE(widget);
+
+  if(event->window == pane->slider) {
+    pane->in_slider = FALSE;
+    gtk_scrollpane_draw_slider(pane);
+  }
+  return FALSE;
 }
 
 static              gint
@@ -739,15 +825,15 @@ gtk_scrollpane_motion_notify(GtkWidget * widget, GdkEventMotion * event)
 
       if (event->window == sp->trough)
 	{
-	  if (width + event->x - sp->x_offset > sp->trough_width - widget->style->klass->xthickness)
-	    x = sp->trough_width - widget->style->klass->xthickness - width;
+	  if (width + event->x - sp->x_offset > sp->trough_width + widget->style->klass->xthickness)
+	    x = sp->trough_width + widget->style->klass->xthickness - width;
 	  else if (event->x - sp->x_offset > widget->style->klass->xthickness)
 	    x = event->x - sp->x_offset;
 	  else
 	    x = widget->style->klass->xthickness;
 
-	  if (height + event->y - sp->y_offset > sp->trough_height - widget->style->klass->ythickness)
-	    y = sp->trough_height - widget->style->klass->ythickness - height;
+	  if (height + event->y - sp->y_offset > sp->trough_height + widget->style->klass->ythickness)
+	    y = sp->trough_height + widget->style->klass->ythickness - height;
 	  else if (event->y - sp->y_offset > widget->style->klass->ythickness)
 	    y = event->y - sp->y_offset;
 	  else
@@ -757,19 +843,19 @@ gtk_scrollpane_motion_notify(GtkWidget * widget, GdkEventMotion * event)
       else if (event->window == sp->slider)
 	{
 
-	  if (x + width + event->x - sp->x_offset > sp->trough_width - widget->style->klass->xthickness)
-	    x = sp->trough_width - widget->style->klass->xthickness - width;
+	  if (x + width + event->x - sp->x_offset > sp->trough_width + widget->style->klass->xthickness)
+	    x = sp->trough_width + widget->style->klass->xthickness - width;
 	  else if (x + event->x - sp->x_offset > widget->style->klass->xthickness)
 	    x += event->x - sp->x_offset;
 	  else
 	    x = widget->style->klass->xthickness;
 
-	  if (y + height + event->y - sp->y_offset > sp->trough_height - widget->style->klass->ythickness)
-	    y = sp->trough_height - widget->style->klass->ythickness - height;
+	  if (y + height + event->y - sp->y_offset > sp->trough_height + widget->style->klass->ythickness)
+	    y = sp->trough_height + widget->style->klass->ythickness - height;
 	  else if (y + event->y - sp->y_offset > widget->style->klass->ythickness)
 	    y += event->y - sp->y_offset;
 	  else
-	    y = widget->style->klass->xthickness;
+	    y = widget->style->klass->ythickness;
 
 	  gdk_window_move(sp->slider, x, y);
 	}
@@ -778,7 +864,7 @@ gtk_scrollpane_motion_notify(GtkWidget * widget, GdkEventMotion * event)
       /* first we check the x adjustment... */
       value = ((sp->Xadjustment->upper - sp->Xadjustment->lower) *
 	       (x + ((gfloat) width) / 2) /
-	       (sp->trough_width - widget->style->klass->xthickness - widget->style->klass->xthickness));
+	       (sp->trough_width));
       if (value + sp->Xadjustment->page_size / 2 < sp->Xadjustment->upper)
 	gtk_adjustment_set_value(sp->Xadjustment, value);
       else
@@ -792,7 +878,7 @@ gtk_scrollpane_motion_notify(GtkWidget * widget, GdkEventMotion * event)
       /* then the y adjustment... */
       value = ((sp->Yadjustment->upper - sp->Yadjustment->lower) *
 	       (y + ((gfloat) height) / 2) /
-	       (sp->trough_height - widget->style->klass->ythickness - widget->style->klass->ythickness));
+	       (sp->trough_height));
       if (value + sp->Yadjustment->page_size / 2 < sp->Yadjustment->upper)
 	gtk_adjustment_set_value(sp->Yadjustment, value);
       else
