@@ -4,9 +4,11 @@
  * Authors: Jonathan Blandford <jrb@redhat.com>
  *          Jaka Mocnik <jaka.mocnik@kiss.uni-lj.si>
  */
+
 #include <config.h>
 #include <gnome.h>
 #include <math.h>
+#include <ctype.h>
 #include <sys/stat.h>
 #include <libgnome/gnome-history.h>
 
@@ -37,6 +39,7 @@ GdkCursor *pan_cursor = NULL;
 static gint width = DEFAULT_WINDOW_WIDTH, height = DEFAULT_WINDOW_HEIGHT;
 
 GList *window_list;
+GList *recent_files_list;
 int active_ggv;
 ggv_prefs prefs_dialog = { NULL };
 
@@ -50,6 +53,13 @@ static gboolean spartan = FALSE;
 static gint     nomenubar = FALSE;
 static gboolean fullscreen = FALSE;
 
+static void free_wwd(window_with_data *wwd)
+{
+        if(wwd->data)
+                g_free(wwd->data);
+        g_free(wwd);
+}
+
 int get_number (const char **geometry)
 {
 	int value = 0;
@@ -155,59 +165,46 @@ char *escape_underlines (char *str)
 void recent_update_menus (ggv_window *ggv, GList *recent_files)
 {
 	GnomeUIInfo *menu;
-        /*	gE_data *data; */
-	gchar *path;
+	gchar *path, *recent_data;
 	int i;
         window_with_data *wwd;
         char *escape;
         
         if (!ggv->show_menus)
                 return;
-        /*
-        g_print("Num recent %d\n", gs_num_recent);
-        */
-	if (gs_num_recent && ggv->recent_instantiated) {
+
+	if (ggv->num_recent > 0)
                 gnome_app_remove_menu_range (GNOME_APP(ggv->main_window), 
-                                             _("_File/"), 4, gs_num_recent + 1);
-                g_print("Removing...\n");
-        }
+                                             "_File/", 4, ggv->num_recent + 1);
 
 	if (recent_files == NULL)
 		return;
 
-
 	/* insert a separator at the beginning */
 	menu = g_malloc0 (2 * sizeof (GnomeUIInfo));
-	path = g_new (gchar, strlen (_("_File")) + strlen ("<Separator>") + 3 );
-	sprintf (path, "%s/%s", _("_File"), "<Separator>");
 	menu->type = GNOME_APP_UI_SEPARATOR;
 
 	(menu + 1)->type = GNOME_APP_UI_ENDOFINFO;
-	gnome_app_insert_menus (GNOME_APP(ggv->main_window), path, menu);
-
+	gnome_app_insert_menus (GNOME_APP(ggv->main_window),
+                                "_File/<Separator>", menu);
 
 	for (i = g_list_length (recent_files) - 1; i >= 0;  i--)
 	{
+                recent_data = (gchar *)g_list_nth_data(recent_files, i);
                 wwd = g_malloc0(sizeof(window_with_data));
 
 		menu = g_malloc0 (2 * sizeof (GnomeUIInfo));
 	
-		/* data = g_malloc0 (sizeof (gE_data));
-                   data->temp1 = g_strdup (g_list_nth_data (recent_files, i)); */
-		/*data->window = window;*/
-		menu->label = g_new (gchar, 
-			strlen (g_list_nth_data (recent_files, i)) + 5);
-
-                escape = escape_underlines ((gchar*)g_list_nth_data (recent_files, i));
-		sprintf (menu->label, "_%i. %s", i+1, escape);
+                escape = escape_underlines (recent_data);
+		menu->label =  g_strdup_printf("_%i. %s", i+1, escape);
                 g_free (escape);
-                
+
 		menu->type = GNOME_APP_UI_ITEM;
 		menu->hint = NULL;
                 
 		menu->moreinfo = (gpointer) recent_callback;
 
-                wwd->data = g_strdup(g_list_nth_data (recent_files, i));
+                wwd->data = g_strdup(recent_data);
                 wwd->ggv = ggv;
 
 		menu->user_data = wwd;
@@ -218,32 +215,33 @@ void recent_update_menus (ggv_window *ggv, GList *recent_files)
 
 		(menu + 1)->type = GNOME_APP_UI_ENDOFINFO;
 	
-		gnome_app_insert_menus (GNOME_APP(ggv->main_window), path, menu);
-		g_free (g_list_nth_data (recent_files, i));
+		gnome_app_insert_menus (GNOME_APP(ggv->main_window),
+                                        "_File/<Separator>", menu);
+
+                gtk_object_set_data_full(GTK_OBJECT(menu->widget), "wwd",
+                                         wwd, (GtkDestroyNotify)free_wwd);
 	}
-	gs_num_recent = g_list_length (recent_files);
-
-        ggv->recent_instantiated = TRUE;
-
-
-
-	g_list_free (recent_files);
+	ggv->num_recent = g_list_length (recent_files);
 }
 
-void recent_update (ggv_window *ggv)
+void recent_update()
 {
-	GList *filelist = NULL;
 	GList *dirlist = NULL;
-	
+        GList *filelist = NULL;
 	GList *gnome_recent_list;
+        GList *ggv_node;
 	GnomeHistoryEntry histentry;
 	char *filename;
 	int i, j;
 	gboolean duplicate = FALSE;
-	
+
 	filelist = NULL;
 	gnome_recent_list = gnome_history_get_recently_used ();
-	
+
+        if(recent_files_list) {
+                g_list_foreach(recent_files_list, (GFunc)g_free, NULL);
+                g_list_free(recent_files_list);
+        }
 
 	if (g_list_length (gnome_recent_list) > 0)                {
                 for (i = g_list_length (gnome_recent_list) - 1; i >= 0; i--) {
@@ -257,8 +255,7 @@ void recent_update (ggv_window *ggv)
 						if (strcmp (histentry->filename, g_list_nth_data (filelist, j)) == 0)
 							filelist = g_list_remove (filelist, g_list_nth_data (filelist, j));
                                 
-				filename = g_malloc0 (strlen (histentry->filename) + 1);
-				strcpy (filename, histentry->filename);
+				filename = g_strdup(histentry->filename);
 				filelist = g_list_append (filelist, filename);
                                 
                                 /* For recent-directories, not yet fully implemented...
@@ -277,12 +274,17 @@ void recent_update (ggv_window *ggv)
 			}
 		}
 	}
+
 	gnome_history_free_recently_used_list (gnome_recent_list);
 
-	gs_num_recent = g_list_length (filelist);
+        recent_files_list = filelist;
 
-	
-	recent_update_menus (ggv, filelist);
+        ggv_node = window_list;
+        while(ggv_node) {
+                recent_update_menus ((ggv_window *)ggv_node->data,
+                                     recent_files_list);
+                ggv_node = ggv_node->next;
+        }
 }
 
 void recent_add(char *filename)
@@ -297,6 +299,7 @@ gboolean load_gs(ggv_window *ggv, char *fname)
 	guint i,n;
 	gchar *texts[3], *flash;
         gint page;
+        gboolean is_structured;
 
         if(ggv->loaded && strcmp(fname, GTK_GS(ggv->gs)->gs_filename) == 0)
                 page = GTK_GS(ggv->gs)->current_page;
@@ -311,8 +314,19 @@ gboolean load_gs(ggv_window *ggv, char *fname)
                 gtk_widget_set_sensitive (ggv->zoomoutbutton, FALSE);
                 gtk_widget_set_sensitive (ggv->zoominbutton, FALSE);
                 gtk_widget_set_sensitive (ggv->printitem, FALSE);
+                gtk_widget_set_sensitive (ggv->printmarked, FALSE);
                 gtk_widget_set_sensitive (ggv->save, FALSE);
                 gtk_widget_set_sensitive (ggv->saveas, FALSE);
+                gtk_widget_set_sensitive (ggv->toggleakt, FALSE);
+                gtk_widget_set_sensitive (ggv->toggleall, FALSE);
+                gtk_widget_set_sensitive (ggv->toggleeven, FALSE);
+                gtk_widget_set_sensitive (ggv->toggleodd, FALSE);
+                gtk_widget_set_sensitive (ggv->clearall, FALSE);
+                gtk_widget_set_sensitive (ggv->toggleakt_pu, FALSE);
+                gtk_widget_set_sensitive (ggv->toggleall_pu, FALSE);
+                gtk_widget_set_sensitive (ggv->toggleeven_pu, FALSE);
+                gtk_widget_set_sensitive (ggv->toggleodd_pu, FALSE);
+                gtk_widget_set_sensitive (ggv->clearall_pu, FALSE);
                 flash = g_strdup_printf(_("Unable to load %s."), fname);
                 error_message(ggv, flash);
                 return FALSE;
@@ -351,10 +365,12 @@ gboolean load_gs(ggv_window *ggv, char *fname)
                               GTK_GS(ggv->gs)->width,GTK_GS(ggv->gs)->height);
 	*/
 
+	is_structured = GTK_GS(ggv->gs)->structured_doc;
+
         if(ggv->pagelist) {
                 gtk_clist_freeze(GTK_CLIST(ggv->pagelist));
                 gtk_clist_clear(GTK_CLIST(ggv->pagelist));
-                if(GTK_GS(ggv->gs)->structured_doc) {
+                if(is_structured) {
                         n = gtk_gs_document_numpages(GTK_GS (ggv->gs));
 
                         if (n > 0) {
@@ -371,14 +387,31 @@ gboolean load_gs(ggv_window *ggv, char *fname)
         gtk_widget_set_sensitive(ggv->zoomoutbutton, TRUE);
         gtk_widget_set_sensitive(ggv->zoominbutton, TRUE);
         gtk_widget_set_sensitive(ggv->printitem, TRUE);
+        gtk_widget_set_sensitive(ggv->printmarked, is_structured);
         gtk_widget_set_sensitive(ggv->reloaditem, TRUE);
+        gtk_widget_set_sensitive(ggv->save, is_structured);
+#if 0
+        /* this is not implemented yet */
         gtk_widget_set_sensitive(ggv->saveas, TRUE);
-        gtk_widget_set_sensitive(ggv->save, TRUE);
+#endif
         set_page_sensitivities(ggv, page);
 
+        gtk_widget_set_sensitive (ggv->toggleakt, is_structured);
+        gtk_widget_set_sensitive (ggv->toggleall, is_structured);
+        gtk_widget_set_sensitive (ggv->toggleeven, is_structured);
+        gtk_widget_set_sensitive (ggv->toggleodd, is_structured);
+        gtk_widget_set_sensitive (ggv->clearall, is_structured);
+        gtk_widget_set_sensitive (ggv->toggleakt_pu, is_structured);
+        gtk_widget_set_sensitive (ggv->toggleall_pu, is_structured);
+        gtk_widget_set_sensitive (ggv->toggleeven_pu, is_structured);
+        gtk_widget_set_sensitive (ggv->toggleodd_pu, is_structured);
+        gtk_widget_set_sensitive (ggv->clearall_pu, is_structured);
+
         flash = g_strdup_printf(_("Successfully loaded %s."), fname);
         flash_message(ggv, flash);
 
+        recent_add(fname);
+
         return TRUE;
 }
 
@@ -388,12 +421,6 @@ void save_marked_pages(ggv_window *ggv, gchar *file)
         gint i;
         FILE *file_handle;
 
-        if(gtk_gs_count_marked_pages(GTK_GS(ggv->gs)) == 0) {
-                error_msg = g_strdup(_("There are no marked pages."));
-                error_message(ggv, error_msg);
-                return;
-        }
-        
         g_print("Pages marked %d\n",
                 gtk_gs_count_marked_pages(GTK_GS(ggv->gs)));
 
@@ -459,7 +486,10 @@ void toggle_pages(ggv_window *ggv, gint what_to_toggle, gint page_number)
 {
         gint n;
 
-        if(!ggv->loaded)
+	g_assert (ggv);
+	g_assert (ggv->gs);
+
+        if((!ggv->loaded) || (!GTK_GS(ggv->gs)->structured_doc))
                 return;
 
         gtk_clist_freeze(GTK_CLIST(ggv->pagelist));
@@ -541,6 +571,7 @@ void create_popup_menus(ggv_window *ggv)
                 {GNOME_APP_UI_ITEM, N_("Recenter Page"), NULL, recenter_page_callback, NULL, NULL,
                  GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
                 {GNOME_APP_UI_SEPARATOR},
+#define PU_PAGE_MENUS_INDEX 6
                 {GNOME_APP_UI_ITEM, N_("Toggle _current page"), NULL, toggle_current_page_callback, NULL ,NULL,
                  GNOME_APP_PIXMAP_NONE, NULL, 0, 0, NULL},
                 {GNOME_APP_UI_ITEM, N_("Toggle _all pages"), NULL, toggle_all_pages_callback, NULL, NULL,
@@ -566,10 +597,21 @@ void create_popup_menus(ggv_window *ggv)
         ggv->prevpu = popupMenu[PU_PREV_PAGE_INDEX].widget;
         ggv->menus_vis_pu = popupMenu[PU_SHOW_MENUS_INDEX].widget;
         ggv->panel_vis_pu = popupMenu[PU_SHOW_PANEL_INDEX].widget;
-        gtk_widget_set_sensitive (GTK_WIDGET(ggv->prevpu),FALSE);
-        gtk_widget_set_sensitive (GTK_WIDGET(ggv->nextpu),FALSE);
+        ggv->toggleakt_pu = popupMenu[PU_PAGE_MENUS_INDEX].widget;
+        ggv->toggleall_pu = popupMenu[PU_PAGE_MENUS_INDEX+1].widget;
+        ggv->toggleeven_pu = popupMenu[PU_PAGE_MENUS_INDEX+2].widget;
+        ggv->toggleodd_pu = popupMenu[PU_PAGE_MENUS_INDEX+3].widget;
+        ggv->clearall_pu = popupMenu[PU_PAGE_MENUS_INDEX+4].widget;
+
+        gtk_widget_set_sensitive (GTK_WIDGET(ggv->prevpu), FALSE);
+        gtk_widget_set_sensitive (GTK_WIDGET(ggv->nextpu), FALSE);
+        gtk_widget_set_sensitive (GTK_WIDGET(ggv->toggleakt_pu), FALSE);
+        gtk_widget_set_sensitive (GTK_WIDGET(ggv->toggleall_pu), FALSE);
+        gtk_widget_set_sensitive (GTK_WIDGET(ggv->toggleeven_pu), FALSE);
+        gtk_widget_set_sensitive (GTK_WIDGET(ggv->toggleodd_pu), FALSE);
+        gtk_widget_set_sensitive (GTK_WIDGET(ggv->clearall_pu), FALSE);
 }
-       
+      
 void create_menus(ggv_window *retval, window_with_data *temp)
 {
         GnomeUIInfo orientationMenu2[] = {
@@ -691,19 +733,34 @@ void create_menus(ggv_window *retval, window_with_data *temp)
         retval->previtem = documentMenu[1].widget;
         retval->reloaditem = fileMenu[2].widget;
         retval->printitem = fileMenu[4].widget;
-        retval->save = fileMenu[5].widget;
-        retval->saveas = fileMenu[6].widget;
+        retval->printmarked = fileMenu[5].widget;
+        retval->save = fileMenu[6].widget;
+        retval->saveas = fileMenu[7].widget;
         retval->menus_vis = settingsMenu[2].widget;
         retval->panel_vis = settingsMenu[3].widget;
+        retval->toggleakt = documentMenu[4].widget;
+        retval->toggleall = documentMenu[5].widget;
+        retval->toggleeven = documentMenu[6].widget;
+        retval->toggleodd = documentMenu[7].widget;
+        retval->clearall = documentMenu[8].widget;
 
         gtk_widget_set_sensitive (GTK_WIDGET(retval->previtem), FALSE);
         gtk_widget_set_sensitive (GTK_WIDGET(retval->nextitem), FALSE);
         gtk_widget_set_sensitive (GTK_WIDGET(retval->printitem), FALSE);
+        gtk_widget_set_sensitive (GTK_WIDGET(retval->printmarked), FALSE);
         gtk_widget_set_sensitive (GTK_WIDGET(retval->reloaditem), FALSE);
+        gtk_widget_set_sensitive (GTK_WIDGET(retval->save), FALSE);
+        gtk_widget_set_sensitive (GTK_WIDGET(retval->saveas), FALSE);
+        gtk_widget_set_sensitive (GTK_WIDGET(retval->toggleakt), FALSE);
+        gtk_widget_set_sensitive (GTK_WIDGET(retval->toggleall), FALSE);
+        gtk_widget_set_sensitive (GTK_WIDGET(retval->toggleeven), FALSE);
+        gtk_widget_set_sensitive (GTK_WIDGET(retval->toggleodd), FALSE);
+        gtk_widget_set_sensitive (GTK_WIDGET(retval->clearall), FALSE);
 
-        gtk_widget_set_sensitive (GTK_WIDGET(fileMenu[5].widget), FALSE);
-        gtk_widget_set_sensitive (GTK_WIDGET(fileMenu[6].widget), FALSE);
-}        
+        if(!recent_files_list)
+                recent_update();
+        recent_update_menus(retval, recent_files_list);
+}
 
 void create_sidebar(ggv_window *retval)
 {
@@ -969,7 +1026,7 @@ ggv_window *create_ggv_window()
         if(retval->show_panel)
                 gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(retval->panel_vis),
                                                 retval->show_panel);
-        retval->recent_instantiated = FALSE;
+
         return retval;
 }
 
@@ -993,17 +1050,15 @@ void open_window(gchar *filename)
 {
         if(filename == NULL) {
                 ggv_window *ggv = create_ggv_window ();
-                active_ggv ++;
+                active_ggv++;
                 window_list = g_list_append(window_list, ggv);
                 gtk_widget_show (ggv->main_window);
-                recent_update(ggv);
-
         }
         else {
                 ggv_window *ggv = create_ggv_window ();
                 gtk_widget_show (ggv->main_window);
 
-                if (load_gs (ggv, filename)) {                                
+                if (load_gs(ggv, filename)) {                                
                         ggv->loaded = TRUE;
                         active_ggv ++;
                         window_list = g_list_append (window_list, ggv);
@@ -1163,6 +1218,8 @@ void open_prefs_dialog(ggv_prefs *pd)
                          GNOME_PAD_SMALL, GNOME_PAD_SMALL);
 	gtk_signal_connect(GTK_OBJECT(pd->gs), "activate",
                            GTK_SIGNAL_FUNC(prefs_changed_callback), pd);
+	gtk_signal_connect(GTK_OBJECT(pd->gs), "changed",
+                           GTK_SIGNAL_FUNC(prefs_changed_callback), pd);
         gtk_widget_show(pd->gs);
         /* scan PDF command */
         label = gtk_label_new(_("Scan PDF"));
@@ -1182,6 +1239,8 @@ void open_prefs_dialog(ggv_prefs *pd)
                          GNOME_PAD_SMALL, GNOME_PAD_SMALL);
 	gtk_signal_connect(GTK_OBJECT(pd->scan_pdf), "activate",
                            GTK_SIGNAL_FUNC(prefs_changed_callback), pd);
+	gtk_signal_connect(GTK_OBJECT(pd->scan_pdf), "changed",
+                           GTK_SIGNAL_FUNC(prefs_changed_callback), pd);
         gtk_widget_show(pd->scan_pdf);
         /* uncompress command */
         label = gtk_label_new(_("Uncompress"));
@@ -1201,6 +1260,8 @@ void open_prefs_dialog(ggv_prefs *pd)
                          GNOME_PAD_SMALL, GNOME_PAD_SMALL);
 	gtk_signal_connect(GTK_OBJECT(pd->uncompress), "activate",
                            GTK_SIGNAL_FUNC(prefs_changed_callback), pd);
+	gtk_signal_connect(GTK_OBJECT(pd->uncompress), "changed",
+                           GTK_SIGNAL_FUNC(prefs_changed_callback), pd);
         gtk_widget_show(pd->uncompress);
 
         label = gtk_label_new(_("Ghostscript"));
@@ -1229,6 +1290,8 @@ void open_prefs_dialog(ggv_prefs *pd)
                          GNOME_PAD_SMALL, GNOME_PAD_SMALL);
 	gtk_signal_connect(GTK_OBJECT(pd->print), "activate",
                            GTK_SIGNAL_FUNC(prefs_changed_callback), pd);
+	gtk_signal_connect(GTK_OBJECT(pd->print), "changed",
+                           GTK_SIGNAL_FUNC(prefs_changed_callback), pd);
         gtk_widget_show(pd->print);
 
         label = gtk_label_new(_("Printing"));
@@ -1248,6 +1311,7 @@ void set_gs_prefs(ggv_window *ggv)
 
         if(gs->antialiased != gs_antialiased && ggv->loaded)
                 redisplay = TRUE;
+
         set_prefs(ggv);
         if(redisplay)
                 reload_callback(NULL, ggv);
@@ -1310,8 +1374,10 @@ struct poptOption options_callback [] = {
 gboolean ggv_file_readable(char *FileName)
 {
         struct stat stat_rec;
+
         if (stat(FileName,&stat_rec) == 0) 
-                return (stat(FileName,&stat_rec) == 0);
+                return TRUE;
+
         return FALSE;
 }
 
@@ -1365,7 +1431,7 @@ int main(int argc, char *argv[])
 
 
                         if (AlterFileName) FileName = AlterFileName;
-                        if (Done) { 
+                        if (Done) {
                                 open_window(FileName);
                                 g_print("opening %s [%s]\n", FileName,AlterFileName);
                         }
