@@ -14,7 +14,6 @@ gchar *gs_cmd = NULL;
 gchar *gs_scan_pdf_cmd = NULL;
 gchar *gs_uncompress_cmd = NULL;
 gchar *gs_print_cmd = NULL;
-gint gs_num_recent=0;
 
 gint gs_def_media;
 gboolean gs_antialiased, gs_override_media;
@@ -45,12 +44,12 @@ void load_prefs(gchar *prefix)
                         g_free(gs_cmd);
                 gs_cmd = s;
         }
-        if(s = gnome_config_get_string("GhostScript/ScanPDF=" GS_PATH " -dNODISPLAY -dQUIET -sPDFname=%s -sDSCname=%s pdf2dsc.ps -c quit")) {
+        if(s = gnome_config_get_string("GhostScript/ScanPDF=" GS_PATH " -dNODISPLAY -dQUIET -sPDFname=\"%s\" -sDSCname=\"%s\" pdf2dsc.ps -c quit")) {
                 if(gs_scan_pdf_cmd)
                         g_free(gs_scan_pdf_cmd);
                 gs_scan_pdf_cmd = s;
         }
-        if(s = gnome_config_get_string("GhostScript/Uncompress=gzip -cd %s > %s")) {
+        if(s = gnome_config_get_string("GhostScript/Uncompress=gzip -cd \"%s\" > %s")) {
                 if(gs_uncompress_cmd)
                         g_free(gs_uncompress_cmd);
                 gs_uncompress_cmd = s;
