@@ -225,19 +225,6 @@ file_fileIsNotUseful(fn)
   return(r);
 }
 
-char *
-file_getTmpFilename(tmpprefix)
-   const gchar *tmpprefix;
-{
-  char *buf = g_malloc(BUFSIZ);
-  if (buf == NULL) {
-	fprintf(stderr, "Fatal Error: Dynamic memory exhausted.\n");
-	exit(-1);
-  }
-  sprintf(buf, "%s%i", tmpprefix, (int) time(0L));
-  return buf;
-}
-
 /* end of patch */
 
 struct document *
@@ -291,7 +278,7 @@ psscan(FILE **fileP, const char *filename, const char *tmpprefix,
 	char *filename_unc;
 	char cmd[BUFSIZ];
 	/*char s[BUFSIZ];*/
-	filename_unc = file_getTmpFilename(tmpprefix);
+ 	filename_unc = tempnam (g_get_tmp_dir(), "unc");
 	sprintf(cmd,cmd_uncompress,filename,filename_unc);
 	fprintf(stderr, "uncompress command:%s\n",cmd);
 	if (system(cmd) || file_fileIsNotUseful(filename_unc)) {
@@ -346,7 +333,7 @@ psscan(FILE **fileP, const char *filename, const char *tmpprefix,
 	char *filename_dsc;
 	char cmd[512];
 	char s[512];
-	filename_dsc=file_getTmpFilename(tmpprefix);
+ 	filename_dsc=tempnam (g_get_tmp_dir(), "dsc");
 	sprintf(cmd,cmd_scan_pdf,filename,filename_dsc);
 	fprintf(stderr, "PDF scan command:%s\n",cmd);
 	if (system(cmd) || file_fileIsNotUseful(filename_dsc)) {
@@ -903,6 +890,32 @@ psscan(FILE **fileP, const char *filename, const char *tmpprefix,
 	doc->lensetup = section_len - line_len;
     }
 
+    /* Added this (Nov. 2, 1999) when I noticed that
+       a Postscript file would load in gv but not in ggv
+
+       dmg@csg.uwaterloo.ca */
+
+    /* BEGIN Windows NT fix ###jp###
+       Mark Pfeifer (pfeiferm%ppddev@comet.cmis.abbott.com) told me
+       about problems when viewing Windows NT 3.51 generated postscript
+       files with gv. He found that the relevant postscript files
+       show important postscript code after the '%%EndSetup' and before
+       the first page comment '%%Page: x y'.
+    */
+    if (doc->beginsetup) {
+      while (!(DSCcomment(line) &&
+	      (iscomment(line+2, "EndSetup") ||
+	      (iscomment(line+2, "Page:") ||
+	       iscomment(line+2, "Trailer") ||
+	       (iscomment(line+2, "EOF"))))) &&
+             (readline(line, sizeof line, file, &position, &line_len))) {
+        section_len += line_len;
+        doc->lensetup = section_len - line_len;
+	doc->endsetup = position;
+      }
+    }
+    /* END Windows NT fix ###jp##*/
+
     /* Individual Pages */
 
     if (beginsection == 0) {
