@@ -128,6 +128,15 @@ char *pscopyuntil(FILE *from, FILE *to, long begin, long end,
 		  const char *comment);
 #else
 char *pscopyuntil();
+#endif
+
+        /* Copy the headers, marked pages, and trailer to fp */
+
+#if NeedFunctionPrototypes
+void pscopydoc(FILE *dest_file, char *src_filename,
+	       struct document *d, int *pagelist);
+#else
+void pscopydoc();
 #endif
 
 #endif
